package com.boman.domain;

import com.boman.domain.annotation.Excel;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.*;

import java.util.List;
import java.util.Map;

/**
 * 常住人口对象 czrk
 *
 * @author ruoyi
 * @date 2022-01-14
 */
@EqualsAndHashCode(callSuper = true)
@Data
@NoArgsConstructor
@AllArgsConstructor
@ToString
public class Czrk extends BaseEntity{
    private static final long serialVersionUID = 1L;

    public static final String IS_DEL_N = "N";
    public static final String IS_DEL_Y = "Y";
    public static final String CZ = "cz";
    public static final String HJ = "hj";

    /**
     * 主键
     */
    private Long id;

    /**
     * 姓名
     */
    @Excel(name = "姓名")
    private String userName;

    /**
     * 性别
     */
    @Excel(name = "性别")
    private String gender;

    /**
     * 年龄
     */
    @Excel(name = "年龄")
    private Integer age;

    /**
     * 出生日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "出生日期", width = 30, dateFormat = "yyyy-MM-dd")
    private String birthday;

    /**
     * 身份证号码
     */
    @Excel(name = "身份证号码")
    private String idCard;

    /**
     * 联系号码
     */
    @Excel(name = "联系号码")
    private String phoneNum;

    /**
     * 人员分类
     */
    @Excel(name = "人员分类")
    private String keyIndustries;

    /**
     * 户籍号
     */
    @Excel(name = "户籍号")
    private String code;

    /**
     * 是否是户主
     */
    @Excel(name = "是否是户主")
    private String houseType;

    /**
     * 认领人
     */
    @Excel(name = "认领人")
    private String rlr;

    /**
     * 认领时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "认领时间", width = 30, dateFormat = "yyyy-MM-dd")
    private java.sql.Date rlTime;

    /**
     * 与户主关系
     */
    @Excel(name = "与户主关系")
    private String yhzgx;

    /** 是否认领 “是”，“否”,判断标准：该人员是否有所属镇 */
    private String isRl;

    /**
     * 当前人所在的最后一级的部门id
     */
    @Excel(name = "当前人所在的最后一级的部门id")
    private Long deptId;


    /**
     * 工作单位
     */
    @Excel(name = "工作单位")
    private String workUnit;

    /**
     * 户籍地的省份
     */
    @Excel(name = "户籍地的省份")
    private Long provinceId;

    /**
     * 户籍地的省份
     */
    @Excel(name = "户籍地的省份")
    private String province;

    /**
     * 户籍地的城市
     */
    @Excel(name = "户籍地的城市")
    private Long cityId;

    /**
     * 户籍地的城市
     */
    @Excel(name = "户籍地的城市")
    private String city;

    /**
     * 户籍地的区
     */
    @Excel(name = "户籍地的区")
    private Long regionId;

    /**
     * 户籍地的区
     */
    @Excel(name = "户籍地的区")
    private String region;

    /**
     * 户籍地的镇
     */
    @Excel(name = "户籍地的镇")
    private Long villageTownsId;

    /**
     * 户籍地的镇
     */
    @Excel(name = "户籍地的镇")
    private String villageTowns;

    /**
     * 户籍地的村
     */
    @Excel(name = "户籍地的村")
    private Long villageId;

    /**
     * 户籍地的村
     */
    @Excel(name = "户籍地的村")
    private String village;

    /**
     * 户籍地的组
     */
    @Excel(name = "户籍地的组")
    private Long villagerGroupId;

    /**
     * 户籍地的组
     */
    @Excel(name = "户籍地的组")
    private String villagerGroup;

    /**
     * 户籍地的详细地址
     */
    @Excel(name = "户籍地的详细地址")
    private String nowIn;

    /**
     * 状态
     */
    @Excel(name = "状态")
    private String status;
    /**
     * 网格id
     */
    private Long gridId;

    /**
     * 是否删除 默认为 "N"
     */
    @Excel(name = "是否删除")
    private String isDel = IS_DEL_N;

    /**
     * 删除原因
     */
    @Excel(name = "删除原因")
    private String deleteReason;
    private String remark;

    private java.sql.Timestamp createTime;
    private java.sql.Timestamp updateTime;
    private String createBy;
    private String updateBy;

    /*************************************************************** Transient ***************************************************************/
    private List<CzrkJzdz> czrkJzdzList;
    private CzrkJzdz czrkJzdz;

    /** 部门id集合 */
    private List<Long> deptIdList;

    /** 现居地列表展示 安徽省安庆市潜山市xx乡xx村xx组 */
    private String nowInSelect;
    /** 户籍地列表展示 安徽省安庆市潜山市xx乡xx村xx组 */
    private String domicileSelect;
    /*** 现居地的详细地址，只留作连表映射用 **/
    private String domicile;

    private String startTime;
    private String endTime;

    /** 查询类型，hj户籍，cz常住,默认是户籍人口 **/
    private String queryType = HJ;
    
    private Long provinceIdXjd;
    private Long cityIdXjd;
    private Long regionIdXjd;
    private Long villageTownsIdXjd;
    private Long villageIdXjd;

    private String provinceXjd;
    private String cityXjd;
    private String regionXjd;
    private String villageTownsXjd;
    private String villageXjd;
}
