package com.boman.web.core.controller;

import com.boman.common.core.utils.poi.ExcelUtil;
import com.boman.common.core.web.controller.BaseController;
import com.boman.common.log.annotation.Log;
import com.boman.common.log.enums.BusinessType;
import com.boman.domain.Czrk;
import com.boman.domain.TableDataInfo;
import com.boman.domain.VaccineInfoOperation;
import com.boman.domain.dto.AjaxResult;
import com.boman.web.core.service.czrk.ICzrkService;
import org.checkerframework.checker.units.qual.C;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * 常住人口Controller
 *
 * @author ruoyi
 * @date 2022-01-14
 */
@RestController
@RequestMapping("/core/czrk")
public class CzrkController extends BaseController {

    @Autowired
    private ICzrkService czrkService;

    /**
     * 查询常住人口列表
     */
//    @PreAuthorize("@ss.hasPermi('system:czrk:list')")
    @GetMapping("/list")
    public TableDataInfo list(Czrk czrk) {
//        startPage();
        List<Czrk> list = czrkService.listByRlry(czrk);
        return getDataTable(list);
    }

    @GetMapping("/listByXz")
    public TableDataInfo listByXz(Czrk czrk) {
        List<Czrk> list = czrkService.listByXz(czrk);
        return getDataTable(list);
    }

    @GetMapping("/listByJs")
    public TableDataInfo listByJs(Czrk czrk) {
        List<Czrk> list = czrkService.listByJs(czrk);
        return getDataTable(list);
    }

    /**
     * 导出常住人口列表
     */
//    @PreAuthorize("@ss.hasPermi('system:czrk:export')")
//    @Log(title = "常住人口", businessType = BusinessType.EXPORT)
//    @PostMapping("/export")
//    public void export(HttpServletResponse response, Czrk czrk) throws IOException {
//        List<Czrk> list = czrkService.selectCzrkList(czrk);
//        ExcelUtil<Czrk> util = new ExcelUtil<>(Czrk.class);
//        util.exportExcel(response, list, "常住人口数据");
//    }

    /**
     * 获取常住人口详细信息
     */
//    @PreAuthorize("@ss.hasPermi('system:czrk:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(czrkService.selectCzrkById(id));
    }

    /**
     * 新增常住人口
     */
//    @PreAuthorize("@ss.hasPermi('system:czrk:add')")
    @Log(title = "常住人口", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Czrk czrk) {
        return toAjax(czrkService.insertCzrk(czrk));
    }

    /**
     * 修改常住人口
     */
//    @PreAuthorize("@ss.hasPermi('system:czrk:edit')")
    @Log(title = "常住人口", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Czrk czrk) {
        return toAjax(czrkService.updateCzrk(czrk));
    }

    /**
     * 功能描述: 删除常住人口，人员认领中的删除也是这个接口
     */
//    @PreAuthorize("@ss.hasPermi('system:czrk:remove')")
    @Log(title = "常住人口", businessType = BusinessType.DELETE)
    @GetMapping("/deleteCzrk")
    public AjaxResult deleteCzrk(Czrk czrk) {
        return toAjax(czrkService.deleteCzrk(czrk));
    }

    /**
     * 功能描述: 回收站的还原功能
     *
     * @return com.boman.domain.dto.AjaxResult
     */
    @GetMapping("/reduction/{id}")
    public AjaxResult reduction(@PathVariable("id") String id) {
        return toAjax(czrkService.reduction(id));
    }

    /**
     * 功能描述: 根据户号查找家庭中所有人员
     *
     * @return com.boman.domain.dto.AjaxResult
     */
    @GetMapping("/family/{code}")
    public AjaxResult family(@PathVariable("code") String code) {
        return AjaxResult.success(czrkService.family(code));
    }

    /**
     * 功能描述: 当前单位：镇，村：7个，组：300个，常住人口：5000人，新增 2人  减少 2人
     *
     * @return com.boman.domain.dto.AjaxResult
     */
    @GetMapping("/stsByCzrk")
    public AjaxResult stsByCzrk() {
        return AjaxResult.success("成功", czrkService.stsByCzrk());
    }

    /**
     * 功能描述: 当前单位：镇，村：7个，组：300个，常住人口：5000人，新增 2人  减少 2人
     * {@link IndexController#stsByCzrkHomePage()}
     *
     * @return com.boman.domain.dto.AjaxResult
     */
    @Deprecated
    @GetMapping("/stsByCzrkHomePage")
    public AjaxResult stsByCzrkHomePage() {
        return AjaxResult.success("成功", czrkService.stsByCzrkHomePage());
    }



    /**
     * 根据身份证号查询用户信息
     */
    @PostMapping("/findHjInfo")
    public AjaxResult findHjInfo(@RequestBody Czrk czrk) {
        Czrk hjInfo = czrkService.findHjInfo(czrk);
        return AjaxResult.success(hjInfo);
    }
}
