package com.boman.web.core.controller;

import com.boman.common.core.utils.poi.ExcelUtil;
import com.boman.common.core.web.controller.BaseController;
import com.boman.common.log.annotation.Log;
import com.boman.common.log.enums.BusinessType;
import com.boman.domain.CzrkJzdz;
import com.boman.domain.TableDataInfo;
import com.boman.domain.dto.AjaxResult;
import com.boman.web.core.service.czrk.ICzrkJzdzService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * 常住人口对应的居住地址Controller
 *
 * @author ruoyi
 * @date 2022-01-14
 */
@RestController
@RequestMapping("/system/jzdz")
public class CzrkJzdzController extends BaseController {
    @Autowired
    private ICzrkJzdzService czrkJzdzService;

    /**
     * 查询常住人口对应的居住地址列表
     */
//    @PreAuthorize("@ss.hasPermi('system:jzdz:list')")
    @GetMapping("/list")
    public TableDataInfo list(CzrkJzdz czrkJzdz) {
        startPage();
        List<CzrkJzdz> list = czrkJzdzService.selectCzrkJzdzList(czrkJzdz);
        return getDataTable(list);
    }

    /**
     * 导出常住人口对应的居住地址列表
     */
//    @PreAuthorize("@ss.hasPermi('system:jzdz:export')")
    @Log(title = "常住人口对应的居住地址", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CzrkJzdz czrkJzdz) throws IOException {
        List<CzrkJzdz> list = czrkJzdzService.selectCzrkJzdzList(czrkJzdz);
        ExcelUtil<CzrkJzdz> util = new ExcelUtil<>(CzrkJzdz.class);
        util.exportExcel(response, list, "常住人口对应的居住地址数据");
    }

    /**
     * 获取常住人口对应的居住地址详细信息
     */
//    @PreAuthorize("@ss.hasPermi('system:jzdz:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(czrkJzdzService.selectCzrkJzdzById(id));
    }

    /**
     * 新增常住人口对应的居住地址
     */
//    @PreAuthorize("@ss.hasPermi('system:jzdz:add')")
    @Log(title = "常住人口对应的居住地址", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CzrkJzdz czrkJzdz) {
        return toAjax(czrkJzdzService.insertCzrkJzdz(czrkJzdz));
    }

    /**
     * 修改常住人口对应的居住地址
     */
//    @PreAuthorize("@ss.hasPermi('system:jzdz:edit')")
    @Log(title = "常住人口对应的居住地址", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CzrkJzdz czrkJzdz) {
        return toAjax(czrkJzdzService.updateCzrkJzdz(czrkJzdz));
    }

    /**
     * 删除常住人口对应的居住地址
     */
//    @PreAuthorize("@ss.hasPermi('system:jzdz:remove')")
    @Log(title = "常住人口对应的居住地址", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(czrkJzdzService.deleteCzrkJzdzByIds(ids));
    }
}
