package com.boman.web.core.controller;

import com.boman.common.core.web.controller.BaseController;
import com.boman.domain.TableDataInfo;
import com.boman.domain.Czrk;
import com.boman.domain.dto.AjaxResult;
import com.boman.web.core.service.czrk.ICzrkService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author shiqian
 * @description 人员认领
 * @date 2022年01月05日 14:26
 **/
@RestController
@RequestMapping("/core/ryrl")
public class RyrlController extends BaseController {

    @Autowired
    private ICzrkService czrkService;

    /**
     * 功能描述: 认领人员list, 如果isDel="Y", 则为回收站的功能，默认是isDel="N"
     *
     * @param czrk czrk
     * @return com.boman.domain.TableDataInfo
     */
    @GetMapping("/list")
    public TableDataInfo list(Czrk czrk) {
        List<Czrk> list = czrkService.listByRlry(czrk);
        return getDataTable(list);
    }

    /**
     * 功能描述: 新增
     *
     * @param czrk czrk
     * @return com.boman.domain.dto.AjaxResult
     */
    @PostMapping("/addRlry")
    public AjaxResult addRlry(@RequestBody Czrk czrk) {
        return czrkService.addRlry(czrk);
    }

    /**
     * 功能描述: 修改 也即 由未认领到认领
     *
     * @param czrk czrk
     * @return com.boman.domain.dto.AjaxResult
     */
    @PostMapping("/editRlry")
    public AjaxResult editRlry(@RequestBody Czrk czrk) {
        return czrkService.editRlry(czrk);
    }

    /**
     * 功能描述: 根据id回显 查czrk和czrk_jzdd
     *
     * @param id id
     * @return com.boman.domain.dto.AjaxResult
     */
    @GetMapping("/getById/{id}")
    public AjaxResult getById(@PathVariable Long id) {
        return AjaxResult.success("成功", czrkService.getById(id));
    }

    /**
     * 功能描述: 小程序   人员新页面，统计全部、已认领、未认领各多少人
     *
     * @return com.boman.domain.dto.AjaxResult
     */
    @GetMapping("/sts")
    public AjaxResult stsByApplet(Long provinceId, Long cityId, Long regionId, Long townId, Long villageId, String userName, String queryType) {
        return AjaxResult.success("成功", czrkService.stsByApplet(provinceId, cityId, regionId, townId, villageId, userName, queryType));
    }


}
