package com.boman.web.core.service.czrk;

import com.alibaba.fastjson.JSONObject;
import com.boman.common.core.utils.DateUtils;
import com.boman.common.core.utils.SecurityUtils;
import com.boman.common.core.utils.StringUtils;
import com.boman.common.core.utils.number.NumberUtils;
import com.boman.common.core.utils.obj.ObjectUtils;
import com.boman.common.core.utils.sql.SqlUtil;
import com.boman.common.core.web.page.PageDomain;
import com.boman.common.core.web.page.TableSupport;
import com.boman.common.redis.service.RedisService;
import com.boman.domain.*;
import com.boman.domain.dto.AjaxResult;
import com.boman.domain.utils.ThreadPoolService;
import com.boman.system.api.RemoteDeptService;
import com.boman.web.core.mapper.CzrkMapper;
import com.boman.web.core.utils.AuthUtils;
import com.boman.web.core.utils.IdCardUtils;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.sql.Date;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import static com.boman.common.core.utils.StringUtils.isNotEmpty;
import static com.boman.common.core.utils.fieldTranslator.IdCardUtils.getSex;
import static com.boman.common.core.utils.obj.ObjectUtils.*;
import static com.boman.common.redis.RedisKey.STS_CZRK_;
import static com.boman.web.core.utils.IdCardUtils.getAge;
import static com.google.common.base.Strings.nullToEmpty;

/**
 * 常住人口Service业务层处理
 *
 * @author ruoyi
 * @date 2022-01-14
 */
@Service
@Slf4j
public class CzrkServiceImpl implements ICzrkService {

    private static JSONObject QIANSHANSHI = null, ZHEN = null, QIANSHANSHI_SINGEL = null;

    @Autowired
    private CzrkMapper czrkMapper;
    @Resource
    private RemoteDeptService remoteDeptService;
    @Resource
    private RedisService redisService;
    @Resource
    private ICzrkJzdzService czrkJzdzService;


    @Override
    public List<Czrk> listByRlry(Czrk czrk) {
        setDeptIdList(czrk);
        setAgeScope(czrk);

        startPage();
        List<Czrk> czrks = czrkMapper.listByRlryRel(czrk);
        handleList1(czrks);
        return czrks;
    }

    private List<CzrkJzdz> getCzrkJzdzs(List<Czrk> czrks) {
//        List<CzrkJzdz> czrkJzdzList = null;
//        // 常住人口需要查 czrk_jzdz
//        if (Czrk.CZ.equals(queryType)) {
        List<String> idCardList = map(czrks, Czrk::getIdCard);
        List<CzrkJzdz> czrkJzdzList = czrkJzdzService.listByIdCard(idCardList);
//        }

        return czrkJzdzList;
    }

    @Override
    public List<Czrk> listByXz(Czrk czrk) {
        setDeptIdList(czrk);

        czrk.setStartTime(DateUtils.getTodayStartStr());
        czrk.setEndTime(DateUtils.getTodayEndStr());
        List<Czrk> czrks = czrkMapper.listByXz(czrk);
        handleList1(czrks);
        return czrks;
    }

    @Override
    public List<Czrk> listByJs(Czrk czrk) {
        setDeptIdList(czrk);

        czrk.setStartTime(DateUtils.getTodayStartStr());
        czrk.setEndTime(DateUtils.getTodayEndStr());
        List<Czrk> czrks = czrkMapper.listByJs(czrk);
        handleList1(czrks);
        return czrks;
    }

    private void handleList(List<Czrk> czrks, String queryType, List<CzrkJzdz> czrkJzdzList) {
        for (Czrk crk : czrks) {
//            if (Czrk.HJ.equals(queryType)) {
            // 户籍地
            String province = nullToEmpty(crk.getProvince()), city = nullToEmpty(crk.getCity()), region = nullToEmpty(crk.getRegion()), villageTowns = nullToEmpty(crk.getVillageTowns()), village = nullToEmpty(crk.getVillage()), villageGroup = nullToEmpty(crk.getVillagerGroup()), nowIn = nullToEmpty(crk.getNowIn());
            crk.setDomicileSelect(province + city + region + villageTowns + village + villageGroup + nowIn);

            // 现居住地
            for (CzrkJzdz czrkJzdz : czrkJzdzList) {
                if (!czrkJzdz.getIdCard().equals(crk.getIdCard())) continue;

                String provinceCz = nullToEmpty(czrkJzdz.getProvince()), cityCz = nullToEmpty(czrkJzdz.getCity()), regionCz = nullToEmpty(czrkJzdz.getRegion()), villageTownsCz = nullToEmpty(czrkJzdz.getTown()), villageCz = nullToEmpty(czrkJzdz.getVillage()), nowInCz = nullToEmpty(czrkJzdz.getNowIn());
                crk.setNowInSelect(provinceCz + cityCz + regionCz + villageTownsCz + villageCz + nowInCz);
                break;
            }
//            }
        }
    }

    private void handleList1(List<Czrk> czrks) {
        if (isEmpty(czrks)) return;
        for (Czrk crk : czrks) {
            // 户籍地
            String province = nullToEmpty(crk.getProvince()), city = nullToEmpty(crk.getCity()), region = nullToEmpty(crk.getRegion()), villageTowns = nullToEmpty(crk.getVillageTowns()), village = nullToEmpty(crk.getVillage()), villageGroup = nullToEmpty(crk.getVillagerGroup()), nowIn = nullToEmpty(crk.getNowIn());
            crk.setDomicileSelect(province + city + region + villageTowns + village + villageGroup + nowIn);

            // 现居住地
            String provinceCz = nullToEmpty(crk.getProvinceXjd()), cityCz = nullToEmpty(crk.getCityXjd()), regionCz = nullToEmpty(crk.getRegionXjd()), villageTownsCz = nullToEmpty(crk.getVillageTownsXjd()), villageCz = nullToEmpty(crk.getVillageXjd()), nowInCz = nullToEmpty(crk.getDomicile());
            crk.setNowInSelect(provinceCz + cityCz + regionCz + villageTownsCz + villageCz + nowInCz);
        }
    }

    private void startPage() {
        PageDomain pageDomain = TableSupport.buildPageRequest();
        Integer pageNum = pageDomain.getPageNum();
        Integer pageSize = pageDomain.getPageSize();
        if (StringUtils.isNotNull(pageNum) && StringUtils.isNotNull(pageSize)) {
            String orderBy = SqlUtil.escapeOrderBySql(pageDomain.getOrderBy());
            PageHelper.startPage(pageNum, pageSize, orderBy);
        }
    }

    private void setAgeScope(Czrk czrk) {
        Map<String, Object> params = czrk.getParams();
        String ageScope = (String) params.get("age");
        if (isNotEmpty(ageScope)) {
            ageScope = ageScope.trim();
            String[] split = ageScope.split("-");
            params.put("startAge", Integer.parseInt(split[0]));
            if (split.length == 2) {
                params.put("endAge", Integer.parseInt(split[1]));
            }
        }
    }

    private void setDeptIdList(Czrk czrk) {
        Long deptId = czrk.getDeptId();
        if (deptId == null) {
            deptId = AuthUtils.getLoginUser().getSysUser().getDeptId();
            //判断是否是管理员
            List<SysRole> roles = AuthUtils.getLoginUser().getSysUser().getRoles();
            if (isNotEmpty(roles)) {
                for (SysRole role : roles) {
                    String roleKey = role.getRoleKey();
                    if ("admin".equals(roleKey) || "city".equals(roleKey) || "yanshi".equals(roleKey)) {
                        return;
                    }
                }
            }
        }

        if (deptId != null) {
            List<SysDept> sysDepts = remoteDeptService.listChildrenDepts(deptId);
            if (isNotEmpty(sysDepts)) {
                czrk.setDeptIdList(map(sysDepts, SysDept::getId));
            }
        }
    }

    private List<Long> getDeptIdList(Long deptId) {
        if (deptId == null) {
            deptId = AuthUtils.getLoginUser().getSysUser().getDeptId();
            //判断是否是管理员
            List<SysRole> roles = AuthUtils.getLoginUser().getSysUser().getRoles();
            if (isNotEmpty(roles)) {
                for (SysRole role : roles) {
                    String roleKey = role.getRoleKey();
                    if ("admin".equals(roleKey) || "city".equals(roleKey)) {
                        return null;
                    }
                }
            }
        }

        if (deptId != null) {
            List<SysDept> sysDepts = remoteDeptService.listChildrenDepts(deptId);
            if (isNotEmpty(sysDepts)) {
                return map(sysDepts, SysDept::getId);
            }
        }

        return null;
    }

    /**
     * 查询常住人口
     *
     * @param id 常住人口主键
     * @return 常住人口
     */
    @Override
    public Czrk selectCzrkById(Long id) {
        return czrkMapper.selectCzrkById(id);
    }

    /**
     * 查询常住人口列表
     *
     * @param czrk 常住人口
     * @return 常住人口
     */
    @Override
    public List<Czrk> selectCzrkList(Czrk czrk) {
        return czrkMapper.selectCzrkList(czrk);
    }

    /**
     * 新增常住人口
     *
     * @param czrk 常住人口
     * @return 结果
     */
    @Override
    public int insertCzrk(Czrk czrk) {
        return czrkMapper.insertCzrk(czrk);
    }

    /**
     * 修改常住人口
     *
     * @param czrk 常住人口
     * @return 结果
     */
    @Override
    public int updateCzrk(Czrk czrk) {
        return czrkMapper.updateCzrk(czrk);
    }

    /**
     * 批量删除常住人口
     *
     * @param czrk 需要删除的常住人口主键和删除原因
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
    public int deleteCzrk(Czrk czrk) {
        czrk.setUpdateBy(AuthUtils.getLoginUser().getSysUser().getUserName());
        if (isEmpty(czrk.getDeleteReason())) {
            log.error("删除原因不能为空, id={}", czrk.getId());
            return 0;
        }

        int delete = czrkMapper.deleteCzrk(czrk);
        Long czrkId = czrk.getId();
        CzrkJzdz czrkJzdz = new CzrkJzdz();
        czrkJzdz.setCzrkId(czrkId);
        czrkJzdz.setStatus("N");
        int update = czrkJzdzService.updateByIdCard(czrkJzdz);
        return 1;
    }

    /**
     * 删除常住人口信息
     *
     * @param id 常住人口主键
     * @return 结果
     */
    @Override
    public int deleteCzrkById(Long id) {
        return czrkMapper.deleteCzrkById(id);
    }

    /**
     * 功能描述: 回收站的还原功能
     *
     * @param id id
     * @return int
     */
    @Override
    public int reduction(String id) {
        if (isEmpty(id)) return 0;

        return czrkMapper.reduction(id);
    }

    @Override
    public List<Czrk> family(String code) {
        if (isEmpty(code)) {
            return Collections.emptyList();
        }

        List<Czrk> czrks = czrkMapper.listByCode(code);
        // 居住地址
        List<CzrkJzdz> czrkJzdzList = getCzrkJzdzs(czrks);
        handleList(czrks, Czrk.HJ, czrkJzdzList);
        return czrks;
    }

    @Override
    public Czrk getById(Long id) {
        if (isEmpty(id)) {
            return null;
        }

        Czrk czrk = czrkMapper.selectCzrkById(id);
        List<CzrkJzdz> czrkJzdzList = czrkJzdzService.listByIdCard(Collections.singletonList(czrk.getIdCard()));
        czrk.setCzrkJzdzList(czrkJzdzList);
        return czrk;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
    public AjaxResult addRlry(Czrk czrk) {
        String username = SecurityUtils.getUsername();
        if (isNotEmpty(czrk.getVillageTowns())) {
            czrk.setRlr(username);
            czrk.setIsRl("是");
            czrk.setRlTime(new Date(System.currentTimeMillis()));
        } else {
            czrk.setIsRl("否");
            czrk.setRlTime(null);
        }

        int count = czrkMapper.countIdCard(czrk.getIdCard());
        if (count > 0) return AjaxResult.error(String.format("该人员[身份证号为: %s]已存在，请勿重复添加", czrk.getIdCard()));

        czrk.setCreateBy(username);
        czrk.setUpdateBy(username);
        count = czrkMapper.insertCzrk(czrk);
        if (count <= 0) return AjaxResult.error();

        List<CzrkJzdz> czrkJzdzList = czrk.getCzrkJzdzList();
        if (isEmpty(czrkJzdzList)) return AjaxResult.error("该人员常住地址未添加");

        int sort = 0, childCount = 0;
        for (CzrkJzdz czrkJzdz : czrkJzdzList) {
            czrkJzdz.setCzrkId(czrk.getId());
            czrkJzdz.setSort(++sort);
            czrkJzdz.setUserName(czrk.getUserName());
            czrkJzdz.setIdCard(czrk.getIdCard());
            czrkJzdz.setCreateBy(username);
            czrkJzdz.setUpdateBy(username);
            childCount += czrkJzdzService.insertCzrkJzdz(czrkJzdz);
        }

        return childCount == czrkJzdzList.size() ? AjaxResult.success() : AjaxResult.error();
    }

    @Override
    public AjaxResult editRlry(Czrk czrk) {
        String username = SecurityUtils.getUsername();
        String isRl = czrk.getIsRl();
        if ("否".equals(isRl)) {
            if (isNotEmpty(czrk.getVillageTowns())) {
                czrk.setRlr(username);
                czrk.setIsRl("是");
                czrk.setRlTime(new Date(System.currentTimeMillis()));
            } else {
                czrk.setIsRl("否");
                czrk.setRlTime(null);
            }
        }

        czrk.setUpdateBy(username);
        int count = czrkMapper.updateCzrk(czrk);
        if (count <= 0) return AjaxResult.error();

        List<CzrkJzdz> czrkJzdzList = czrk.getCzrkJzdzList();
        if (isEmpty(czrkJzdzList)) return AjaxResult.error("该人员常住地址未添加");

        czrkJzdzService.deleteByCzrkId(czrk.getId());
        int sort = 0, childCount = 0;
        for (CzrkJzdz czrkJzdz : czrkJzdzList) {
            czrkJzdz.setCzrkId(czrk.getId());
            czrkJzdz.setSort(++sort);
            czrkJzdz.setUserName(czrk.getUserName());
            czrkJzdz.setIdCard(czrk.getIdCard());
            czrkJzdz.setCreateBy(username);
            czrkJzdz.setUpdateBy(username);
            childCount += czrkJzdzService.insertCzrkJzdz(czrkJzdz);
        }

        return childCount == czrkJzdzList.size() ? AjaxResult.success() : AjaxResult.error();
    }

    @Override
    public JSONObject stsByApplet(Long provinceId, Long cityId, Long regionId, Long townId, Long villageId, String userName, String queryType) {
        Long areaId = null;
        int type = 0, yrl = 0, wrl = 0;
        if (ObjectUtils.isNotEmpty(provinceId)) {
            areaId = provinceId;
            type = 1;
        } else if (ObjectUtils.isNotEmpty(cityId)) {
            areaId = cityId;
            type = 2;
        } else if (ObjectUtils.isNotEmpty(regionId)) {
            areaId = regionId;
            type = 3;
        } else if (ObjectUtils.isNotEmpty(townId)) {
            areaId = townId;
            type = 4;
        } else if (ObjectUtils.isNotEmpty(villageId)) {
            areaId = villageId;
            type = 5;
        }

        if (Czrk.HJ.equals(queryType)) {
            // 户籍
            yrl = czrkMapper.stsByYrl(type, areaId, userName);
            wrl = czrkMapper.stsByWrl(type, areaId, userName);
        } else {
            // 常住
            List<Czrk> rls = czrkMapper.stsByAppletCz(type, areaId, userName);
            for (Czrk czrk : rls) {
                if ("是".equals(czrk.getIsRl())) {
                    yrl++;
                } else if ("否".equals(czrk.getIsRl())) {
                    wrl++;
                }
            }
        }

        JSONObject result = new JSONObject(3);
        result.put("yrl", yrl);
        result.put("wrl", wrl);
        result.put("total", wrl + yrl);
        return result;
    }

    /**
     * 功能描述: 当前单位：镇，村：7个，组：300个，常住人口：5000人，新增 2人  减少 2人
     *
     * @return com.alibaba.fastjson.JSONObject
     */
    @Override
    public JSONObject stsByCzrk() {
        JSONObject result = new JSONObject(16);
        SysUser sysUser = AuthUtils.getLoginUser().getSysUser();

        Long deptId = sysUser.getDeptId(); // 当前登陆人的deptId
        result.put("dqdw", sysUser.getDept().getDeptName());  // 当前单位

        String startTime = DateUtils.getTodayStartStr();
        String endTime = DateUtils.getTodayEndStr();

        // 市领导
        List<String> roleKeySets = map(sysUser.getRoles(), SysRole::getRoleKey);
        if (roleKeySets.contains("city") || roleKeySets.contains("admin") || roleKeySets.contains("yanshi")) {
            JSONObject jsonObject = redisService.getCacheObject(packRedisKey("single:" + deptId));
            if (isNotEmpty(jsonObject)) {
                QIANSHANSHI_SINGEL = jsonObject;
                return jsonObject;
            }

            ThreadPoolService.execute(() -> {
                Long areaId = 340882000000L;
                List<SysDept> allDepts = remoteDeptService.listChildrenDepts(1L);
                List<SysDept> townsDepts = new ArrayList<>(16);
                for (SysDept allDept : allDepts) {
                    if (isEmpty(allDept.getParentId())) continue;
                    if (allDept.getParentId() == (1L)) {
                        townsDepts.add(allDept);
                    }
                }
                result.put("zhen", townsDepts.size()); // 镇

                List<SysDept> cunDepts = new ArrayList<>(16);
                for (SysDept townsDept : townsDepts) {
                    for (SysDept allDept : allDepts) {
                        if (isEmpty(allDept.getParentId())) continue;
                        if (townsDept.getId().equals(allDept.getParentId())) {
                            cunDepts.add(allDept);
                        }
                    }
                }
                result.put("cun", isEmpty(cunDepts) ? 0 : cunDepts.size()); //村
//
                List<SysDept> zuDept = new ArrayList<>(16);
                for (SysDept cunDept : cunDepts) {
                    for (SysDept allDept : allDepts) {
                        if (cunDept.getId().equals(allDept.getParentId())) {
                            zuDept.add(allDept);
                        }
                    }
                }

                result.put("zu", isEmpty(zuDept) ? 0 : zuDept.size()); // 组

                // 常住人口信息 areaId 为潜山市的为常住人口
                int type = 3;
                int czzrs = czrkMapper.countCzrk(type, areaId); // 常住人口
                result.put("czzrs", czzrs);

                int czxz = czrkMapper.countByCzXz(startTime, endTime, type, areaId); // 新增
                int czjs = czrkMapper.countByCzJs(startTime, endTime, type, areaId);// 减少
                result.put("czxz", czxz);
                result.put("czjs", czjs);

                // condition.setIsRl(null); // 查全部
                type = 0;
                int zrs = czrkMapper.countCzrk(type, areaId);
                String czbfb = NumberUtils.percent(czzrs, zrs);
                result.put("czbfb", czbfb);
                result.put("zrs", zrs);

                // 户籍人口信息
                // 户籍总人数
                List<Czrk> hjzrsList = czrkMapper.countHj(startTime, endTime, type, areaId);
                int hjzrs= isEmpty(hjzrsList) ? 0 : hjzrsList.size();
                result.put("hjzrs", hjzrs);

                // 户籍人口信息 新增 减少
                type = 3; // regionId
                List<Czrk> hjxzs = czrkMapper.countByHjXz(startTime, endTime, type, areaId); // 新增
                List<Czrk> hjjss = czrkMapper.countByHjJs(startTime, endTime, type, areaId);// 减少
                result.put("hjxz", isEmpty(hjxzs) ? 0 : hjxzs.size());
                result.put("hjjs", isEmpty(hjjss) ? 0 : hjjss.size());

                // 户籍人口信息 百分比
                String hjbfb = NumberUtils.percent(hjzrs, zrs);
                result.put("hjbfb", hjbfb);
                setIntoRedis(packRedisKey("single:" + deptId), result);
                QIANSHANSHI_SINGEL = result;
            });

            return QIANSHANSHI_SINGEL;
        } else if (roleKeySets.contains("sys:town")) {
            JSONObject jsonObject = redisService.getCacheObject(packRedisKey("town:" + deptId));
            if (isNotEmpty(jsonObject)) {
                return jsonObject;
            }

            townSts(result, deptId, sysUser.getDept().getDeptName(), true);
//            redisService.setCacheObject(packRedisKey("town:" + deptId), result, 1L, TimeUnit.DAYS);
            setIntoRedis(packRedisKey("town:" + deptId), result);
        }

        return result;
    }

    /**
     * 功能描述: 项目首页统计信息，
     * 当前单位：镇，村：7个，组：300个，常住人口：5000人，新增 2人  减少 2人
     *
     * @return com.boman.domain.dto.AjaxResult
     */
    @Override
    public JSONObject stsByCzrkHomePage() {
        JSONObject result = new JSONObject(16);
        // 当前登陆人的deptId
        SysUser sysUser = AuthUtils.getLoginUser().getSysUser();
        String deptName = sysUser.getDept().getDeptName();
        result.put("dqdw", deptName); // 当前单位

        List<String> roleKeySets = map(sysUser.getRoles(), SysRole::getRoleKey);

        String todayStart = DateUtils.getTodayStartStr();
        String todayEnd = DateUtils.getTodayEndStr();
        Long deptId = sysUser.getDeptId();

        // 市领导 todo 此处不应该写死
        if (roleKeySets.contains("city") || roleKeySets.contains("admin") || roleKeySets.contains("yanshi")) {
            JSONObject jsonObject = redisService.getCacheObject(packRedisKey(deptId + ""));
            if (isNotEmpty(jsonObject)) {
                QIANSHANSHI = jsonObject;
                return jsonObject;
            }

            ThreadPoolService.execute(() -> {
                Long areaId = 340882000000L;
                List<SysDept> allDepts = remoteDeptService.listChildrenDepts(1L);
                List<SysDept> townsDepts = new ArrayList<>(16);
                for (SysDept allDept : allDepts) {
                    if (isEmpty(allDept.getParentId())) continue;
                    if (allDept.getParentId() == (1L)) {
                        townsDepts.add(allDept);
                    }
                }
                result.put("zhen", townsDepts.size()); // 镇

                List<SysDept> cunDepts = new ArrayList<>(16);
                for (SysDept townsDept : townsDepts) {
                    for (SysDept allDept : allDepts) {
                        if (isEmpty(allDept.getParentId())) continue;
                        if (townsDept.getId().equals(allDept.getParentId())) {
                            cunDepts.add(allDept);
                        }
                    }
                }
                result.put("cun", isEmpty(cunDepts) ? 0 : cunDepts.size());

                // 常住人口信息 areaId 为潜山市的为常住人口
                int type = 3;
                int czzrs = czrkMapper.countCzrk(type, areaId); // 常住人口
                result.put("czzrs", czzrs);

                int czxz = czrkMapper.countByCzXz(todayStart, todayEnd, type, areaId); // 新增
                int czjs = czrkMapper.countByCzJs(todayStart, todayEnd, type, areaId);// 减少
                result.put("czxz", czxz);
                result.put("czjs", czjs);

                // condition.setIsRl(null); // 查全部
                type = 0;
                int zrs = czrkMapper.countCzrk(type, areaId);
                String czbfb = NumberUtils.percent(czzrs, zrs);
                result.put("czbfb", czbfb);
                result.put("zrs", zrs);

                // 户籍人口信息
                // 户籍总人数
                List<Czrk> hjzrsList = czrkMapper.countHj(todayStart, todayEnd, type, areaId);
                int hjzrs= isEmpty(hjzrsList) ? 0 : hjzrsList.size();
                result.put("hjzrs", hjzrs);

                // 户籍人口信息 新增 减少
                type = 3; // regionId
                List<Czrk> hjxzs = czrkMapper.countByHjXz(todayStart, todayEnd, type, areaId); // 新增
                List<Czrk> hjjss = czrkMapper.countByHjJs(todayStart, todayEnd, type, areaId);// 减少
                result.put("hjxz", isEmpty(hjxzs) ? 0 : hjxzs.size());
                result.put("hjjs", isEmpty(hjjss) ? 0 : hjjss.size());

                // 户籍人口信息 百分比
                String hjbfb = NumberUtils.percent(hjzrs, zrs);
                result.put("hjbfb", hjbfb);

                // 以各个镇为单位单独处理 townsDepts
                ArrayList<JSONObject> towsDataList = new ArrayList<>(20);
                for (SysDept townsDept : townsDepts) {
                    JSONObject townsResult = new JSONObject(16);
                    townSts(townsResult, townsDept.getId(), townsDept.getDeptName(), false);
                    towsDataList.add(townsResult);
                }

                result.put("townDatas", towsDataList);
//                redisService.setCacheObject(packRedisKey(deptId + ""), result, 1L, TimeUnit.DAYS);
                setIntoRedis(packRedisKey(deptId + ""), result);
                QIANSHANSHI = result;
            });

            return QIANSHANSHI;
        } else if (roleKeySets.contains("sys:town")) {
            JSONObject jsonObject = redisService.getCacheObject(packRedisKey("town:" + deptId));
            if (isNotEmpty(jsonObject)) {
                ZHEN = jsonObject;
                return jsonObject;
            }

            ThreadPoolService.execute(() -> {
                townSts(result, deptId, deptName, true);
//                redisService.setCacheObject(packRedisKey("town:" + deptId), result, 1L, TimeUnit.DAYS);
                setIntoRedis(packRedisKey("town:" + deptId), result);
            });

            return ZHEN;
        }

        return result;
    }

    private void townSts(JSONObject result, Long deptId, String deptName, boolean percent) {
        String startTime = DateUtils.getTodayStartStr();
        String endTime = DateUtils.getTodayEndStr();
        Long areaId = deptId;
        // 乡镇以下的所有部门
        List<SysDept> depts = remoteDeptService.listChildrenDepts(deptId);
        List<SysDept> cunDepts = new ArrayList<>(16);
        for (SysDept dept : depts) {
            if (isEmpty(dept.getParentId())) continue;
            if (deptId.equals(dept.getParentId())) {
                cunDepts.add(dept);
            }
        }
        result.put("cun", isEmpty(cunDepts) ? 0 : cunDepts.size());

        List<SysDept> zuDept = new ArrayList<>(16);
        for (SysDept cunDept : cunDepts) {
            for (SysDept dept : depts) {
                if (cunDept.getId().equals(dept.getParentId())) {
                    zuDept.add(dept);
                }
            }
        }

        result.put("zu", isEmpty(zuDept) ? 0 : zuDept.size());

        // 常住人口信息 areaId 为潜山市的为常住人口
        int type = 0;
        int czzrs = czrkMapper.countCzrk(type, areaId); // 常住人口
        result.put("czzrs", czzrs);

        int zrs = 0;
        // 常住人口占总人口的百分比
        if (BooleanUtils.isTrue(percent)) {
             // 查全部
            zrs = czrkMapper.countCzrk(type, areaId);
            String czbfb = NumberUtils.percent(czzrs, zrs);
            result.put("czbfb", czbfb);
        }

        type = 4;
        int czxz = czrkMapper.countByCzXz(startTime, endTime, type, areaId);
        int czjs = czrkMapper.countByCzJs(startTime, endTime, type, areaId);
        result.put("czxz", czxz);
        result.put("czjs", czjs);

        result.put("zhen", deptName);

        // 户籍人口信息
        List<Czrk> hjxzs = czrkMapper.countByHjXz(startTime, endTime, type, areaId); // 新增
        List<Czrk> hjjss = czrkMapper.countByHjJs(startTime, endTime, type, areaId);// 减少
        result.put("hjxz", isEmpty(hjxzs) ? 0 : hjxzs.size());
        result.put("hjjs", isEmpty(hjjss) ? 0 : hjjss.size());

        // 户籍总人数
        List<Czrk> hjzrsList = czrkMapper.countHj(startTime, endTime, type, areaId);
        int hjzrs= isEmpty(hjzrsList) ? 0 : hjzrsList.size();
        result.put("hjzrs", hjzrs);
        String hjbfb = NumberUtils.percent(hjzrs, zrs);
        result.put("hjbfb", hjbfb);
    }

    @Override
    public JSONObject hjczLine() {
        // 统计范围 从今天往前推7天
        java.util.Date startTimeDate = DateUtils.getTodayStart(), endTimeDate = DateUtils.addDays(startTimeDate, -7);
        String startTime = DateUtils.dateTime1(startTimeDate), endTime = DateUtils.dateTime1(endTimeDate);
        SysUser sysUser = AuthUtils.getLoginUser().getSysUser();
        int type = 0;
        Long areaId = null;

        // 市领导 todo 此处不应该写死
        List<String> roleKeySets = map(sysUser.getRoles(), SysRole::getRoleKey);
        if (roleKeySets.contains("city") || roleKeySets.contains("admin") || roleKeySets.contains("yanshi")) {
            type = 3;
            areaId = 340882000000L;
        } else if (roleKeySets.contains("sys:town")) {
            type = 4;
            areaId = sysUser.getDeptId();
        }

        List<String> dateList = DateUtils.splitDate(startTimeDate, endTimeDate);

        // 常住人口信息
        List<Czrk> czxzs = czrkMapper.listByCzXz(startTime, endTime, type, areaId);
        List<Czrk> czjss = czrkMapper.listByCzJs(startTime, endTime, type, areaId);

        // 户籍人口信息
        List<Czrk> hjxzs = czrkMapper.countByHjXz(startTime, endTime, type, areaId); // 新增
        List<Czrk> hjjss = czrkMapper.countByHjJs(startTime, endTime, type, areaId);// 减少

//        String lastDate = dateList.get(dateList.size() - 1);
        for (String week : dateList) {
            JSONObject temp = new JSONObject();
            temp.put("time", week);
            for (Czrk czxz : czxzs) {
                Timestamp createTime = czxz.getCreateTime();

            }

            for (Czrk czjs : czjss) {

            }

        }

        return null;
    }

    /**
     * 根据身份证号查询用户信息
     * @param czrk
     * @return
     */
    @Override
    public Czrk findHjInfo(Czrk czrk) {
        Czrk hjInfo = czrkMapper.findHjInfo(czrk);
        if (hjInfo == null) {
            String idCard = czrk.getIdCard();
            if (StringUtils.isNotBlank(idCard)) {
                //根据身份证提取出生日和性别,年龄
                String sex = getSex(idCard);
                Integer age = getAge(idCard);
                String birthday = IdCardUtils.getBirthday(idCard);
                czrk.setAge(age);
                czrk.setGender(sex);
                czrk.setBirthday(birthday);
                return czrk;
            }
        }
        return hjInfo;
    }

    private String packRedisKey(String deptId) {
        return STS_CZRK_ + deptId + ":" + DateUtils.getDate();
    }

    private void setIntoRedis(String key, Object result) {
        redisService.setCacheObject(key, result, 1L, TimeUnit.HOURS);
    }
}
